@@ -46,8 +46,10 @@ BOOST_AUTO_TEST_CASE(engine_version)
 {
     RubberBandStretcher s2(44100, 1, RubberBandStretcher::OptionEngineFaster);
     BOOST_TEST(s2.getEngineVersion() == 2);
+    BOOST_TEST(s2.getProcessSizeLimit() == 524288);
     RubberBandStretcher s3(44100, 1, RubberBandStretcher::OptionEngineFiner);
     BOOST_TEST(s3.getEngineVersion() == 3);
+    BOOST_TEST(s3.getProcessSizeLimit() == 524288);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_unchanged_offline_faster)
@@ -233,6 +235,7 @@ static vector<float> process_realtime(RubberBandStretcher &stretcher,
                                       const vector<float> &in,
                                       int nOut,
                                       int bs,
+                                      bool roundUpProcessSize,
                                       bool printDebug)
 {
     int n = in.size();
@@ -270,9 +273,20 @@ static vector<float> process_realtime(RubberBandStretcher &stretcher,
             } else if (available == 0) { // need to provide more input
                 int required = stretcher.getSamplesRequired();
                 BOOST_TEST(required > 0); // because available == 0
-                int toProcess = std::min(required, n - inOffset);
+                int toProcess = required;
+                if (roundUpProcessSize) {
+                    // We sometimes want to explicitly test passing
+                    // large blocks to process, longer than
+                    // getSamplesRequired indicates
+                    toProcess = std::max(required, bs);
+                }
+                bool final = false;
+                if (toProcess >= n - inOffset) {
+                    toProcess = n - inOffset;
+                    final = true;
+                }
                 const float *const source = in.data() + inOffset;
-                bool final = (toProcess < required);
+//                cerr << "toProcess = " << toProcess << ", inOffset = " << inOffset << ", n = " << n << ", required = " << required << ", outOffset = " << outOffset << ", obtained = " << obtained << ", bs = " << bs << ", final = " << final << endl;
                 stretcher.process(&source, toProcess, final);
                 inOffset += toProcess;
                 BOOST_TEST(stretcher.available() > 0);
@@ -310,24 +324,27 @@ static vector<float> process_realtime(RubberBandStretcher &stretcher,
 static void sinusoid_realtime(RubberBandStretcher::Options options,
                               double timeRatio,
                               double pitchScale,
-                              bool printDebug)
+                              int bs = 512,
+                              bool roundUpProcessSize = false,
+                              bool printDebug = false)
 {
     int n = (timeRatio < 1.0 ? 80000 : 40000);
+    if (n < bs * 2) n = bs * 2;
     int nOut = int(ceil(n * timeRatio));
     float freq = 441.f;
     int rate = 44100;
-    int bs = 512;
 
     // This test simulates block-by-block realtime processing with
     // latency compensation, and checks that the output is all in the
     // expected place
 
     RubberBandStretcher stretcher(rate, 1, options, timeRatio, pitchScale);
-    stretcher.setMaxProcessSize(bs);
 
     if (printDebug) {
         stretcher.setDebugLevel(2);
     }
+
+    stretcher.setMaxProcessSize(bs);
     
     // The input signal is a fixed frequency sinusoid that steps up in
     // amplitude every 1/10 of the total duration - from 0.1 at the
@@ -341,7 +358,8 @@ static void sinusoid_realtime(RubberBandStretcher::Options options,
         in[i] = sample;
     }
 
-    vector<float> out = process_realtime(stretcher, in, nOut, bs, printDebug);
+    vector<float> out = process_realtime(stretcher, in, nOut, bs,
+                                         roundUpProcessSize, printDebug);
         
     // Step through the output signal in chunk of 1/20 of its duration
     // (i.e. a rather arbitrary two per expected 0.1 increment in
@@ -431,24 +449,21 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_samepitch_realtime_finer)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime,
-                      8.0, 1.0,
-                      false);
+                      8.0, 1.0);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_samepitch_realtime_finer)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime,
-                      0.5, 1.0,
-                      false);
+                      0.5, 1.0);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_hqpitch)
@@ -456,8 +471,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_hqpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighQuality,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_hcpitch)
@@ -465,16 +479,14 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_hcpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_hqpitch)
@@ -482,8 +494,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_hqpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighQuality,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_hcpitch)
@@ -491,16 +502,14 @@ BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_hcpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_hqpitch)
@@ -508,8 +517,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_hqpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighQuality,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_hcpitch)
@@ -517,8 +525,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_hcpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_samepitch_realtime_finer_short)
@@ -526,8 +533,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_samepitch_realtime_finer_short)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort,
-                      8.0, 1.0,
-                      false);
+                      8.0, 1.0);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_samepitch_realtime_finer_short)
@@ -535,8 +541,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_fast_samepitch_realtime_finer_short)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort,
-                      0.5, 1.0,
-                      false);
+                      0.5, 1.0);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_short)
@@ -544,8 +549,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_short)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_short_hcpitch)
@@ -554,8 +558,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_finer_short_hcpitch)
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_short)
@@ -563,8 +566,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_short)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_hcpitch_short)
@@ -573,8 +575,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_finer_hcpitch_short)
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_short)
@@ -582,8 +583,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_short)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_short_hcpitch)
@@ -592,32 +592,28 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_finer_short_hcpitch)
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionWindowShort |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_samepitch_realtime_faster)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime,
-                      8.0, 1.0,
-                      false);
+                      8.0, 1.0);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_samepitch_realtime_faster)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime,
-                      0.5, 1.0,
-                      false);
+                      0.5, 1.0);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_faster)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_faster_hqpitch)
@@ -625,8 +621,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_faster_hqpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighQuality,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_faster_hcpitch)
@@ -634,16 +629,14 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_higher_realtime_faster_hcpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      4.0, 1.5,
-                      false);
+                      4.0, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_faster)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_faster_hqpitch)
@@ -651,8 +644,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_faster_hqpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighQuality,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_faster_hcpitch)
@@ -660,16 +652,14 @@ BOOST_AUTO_TEST_CASE(sinusoid_fast_higher_realtime_faster_hcpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      0.5, 1.5,
-                      false);
+                      0.5, 1.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_faster)
 {
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_faster_hqpitch)
@@ -677,8 +667,7 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_faster_hqpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighQuality,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
 }
 
 BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_faster_hcpitch)
@@ -686,8 +675,71 @@ BOOST_AUTO_TEST_CASE(sinusoid_slow_lower_realtime_faster_hcpitch)
     sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
                       RubberBandStretcher::OptionProcessRealTime |
                       RubberBandStretcher::OptionPitchHighConsistency,
-                      8.0, 0.5,
-                      false);
+                      8.0, 0.5);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_faster)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      4.0, 0.5,
+                      80000, true);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_faster_stretch)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      2.0, 1.0,
+                      80000, true);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_faster_shrink)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      0.8, 1.0,
+                      80000, true);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_faster_higher)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      1.0, 2.0,
+                      80000, true);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_faster_lower)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFaster |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      1.0, 0.5,
+                      80000, true);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_finer)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      4.0, 0.5,
+                      80000, true);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_finer_stretch)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      2.0, 1.0,
+                      80000, true);
+}
+
+BOOST_AUTO_TEST_CASE(sinusoid_realtime_long_blocksize_finer_shift)
+{
+    sinusoid_realtime(RubberBandStretcher::OptionEngineFiner |
+                      RubberBandStretcher::OptionProcessRealTime,
+                      1.0, 0.5,
+                      80000, true);
 }
 
 BOOST_AUTO_TEST_CASE(impulses_2x_offline_faster)
@@ -925,7 +977,8 @@ static void impulses_realtime(RubberBandStretcher::Options options,
     in[9900] = 1.f;
     in[9901] = -1.f;
 
-    vector<float> out = process_realtime(stretcher, in, nOut, bs, printDebug);
+    vector<float> out = process_realtime(stretcher, in, nOut, bs,
+                                         false, printDebug);
 
     int peak0 = -1, peak1 = -1, peak2 = -1;
     float max;
@@ -948,14 +1001,14 @@ static void impulses_realtime(RubberBandStretcher::Options options,
     // These limits aren't alarming, but it may be worth tightening
     // them and and taking a look at the waveforms
     
-    BOOST_TEST(peak0 < int(ceil(200 * timeRatio)));
+    BOOST_TEST(peak0 < int(ceil(210 * timeRatio)));
     BOOST_TEST(peak0 > int(ceil(50 * timeRatio)));
 
     BOOST_TEST(peak1 < int(ceil(5070 * timeRatio)));
     BOOST_TEST(peak1 > int(ceil(4640 * timeRatio)));
 
     BOOST_TEST(peak2 < int(ceil(9970 * timeRatio)));
-    BOOST_TEST(peak2 > int(ceil(9770 * timeRatio)));
+    BOOST_TEST(peak2 > int(ceil(9670 * timeRatio)));
 
     if (printDebug) {
         std::cout << "#sample\tV" << std::endl;
@@ -1392,8 +1445,8 @@ static void with_resets(RubberBandStretcher::Options options,
             for (int i = 0; i < nActual; ++i) {
                 BOOST_TEST(out[i] == outRef[i]);
                 if (out[i] != outRef[i]) {
-                    std::cerr << "Failure at index " << i << " in run "
-                              << run << std::endl;
+                    std::cerr << "Failure at index " << i << " of "
+                              << nActual << " in run " << run << std::endl;
                     break;
                 }
             }
